/*
 * Per prima cosa ordiniamo l'array che contiene i valori inseriti
 * Quando possibile, sfruttiamo il fatto che l'array e' ordinato per 
 * ottimizzare le varie funzioni.
 */

#include <stdio.h>

#define SIZE 10 // deve essere almeno 1

// == dichiarazione funzioni ==================================================
void leggi(int[], int);
void stampa(int [], int);
void ordina(int[], int);
int minimo(int []);
int massimo(int [], int);
double media(int[], int);
int moda(int[], int);
double mediana(int[], int);

// == definizione funzioni ====================================================

int main(void) {
  int a[SIZE];
  int i;

	leggi( a, SIZE );
  printf("Valori inseriti: ");
  stampa(a, SIZE);

	ordina(a, SIZE);
  printf("Valori ordinati: ");
  stampa(a, SIZE);

  printf("Il minimo e': %i\n", minimo(a));
  printf("Il massimo e': %i\n", massimo(a, SIZE));
  printf("La media e': %2lf\n", media(a, SIZE));
  printf("La moda e': %i\n", moda(a, SIZE));
  printf("La mediana e': %lf\n", mediana(a, SIZE));
}


/* Funzione che legge size elementi positivi e li memorizza nell'array a.
 */
void leggi(int a[], int size) {
  int i = 0;
  int n;

  do {
    printf("Numero %i: ", i+1);
    scanf("%i", &n);
    if (n >= 0) {
      a[i] = n;
      i++;
    }
  } while (i < size);
}

/* Funzione che stampa gli elementi dell'array a.
 */
void stampa(int a[], int size) {
  int i;
  
  printf("[ ");
  for (i = 0; i < size; i++)
    printf("%i ", a[i]);
  printf(" ]\n");
}

/* Funzione che ordina l'array a.
 * Implementazione alternativa di min_sort.
 */
void ordina(int a[], int size) {
  int i;       // limite delle posizioni ordinate
  int j;       // indice per la ricerca del minimo
  int tmp;     // variabile temporanea per lo scambio

  for (i = 0; i < size; i++)
    // Ad ogni iterazione, cerchiamo il minimo da mettere in a[i].
    // Quando troviamo un elemento minore, lo scambiamo direttamente in a[i]
    for (j = i + 1; j < size; j++)
      if (a[i] > a[j]) { 
        tmp = a[i];
        a[i] = a[j];
        a[j] = tmp;
      } 
}

/* Funzione che restituisce i massimo di un  array *ordinato*
 * PRECONDIZIONE: array a ordinato
 */
int massimo(int a[], int size) {
  return a[size - 1];
 }

/* Funzione che restituisce i massimo di un  array *ordinato*
 * PRECONDIZIONE: array a ordinato con almeno un elemento
 */
 int minimo(int a[]) {
  return a[0];
}

/* Funzione che calcola la media degli elementi in a.
 */
double media(int a[], int size) {
  int i; 
  double s = 0.0;

  for(i = 0; i < size; i++)
    s += a[i];
  return s / size;
}

/* Funzione che calcola la moda degli elementi in un array *ordinato*.
 * PRECONDIZIONE: array a ordinato.
 */
int moda(int a[], int size) {
  int m;     // moda
  int f = 0; // frequenza di m
  int i, j;

  for (i = 0; i <size; i = j) {
    for (j = i + 1; j < size && a[i] == a[j]; j++); // avanziamo con j finche' troviamo valori uguali
    if (j - i > f) { // controlliamo se il numero di valori uguali supera la frequenza
			m = a[i];
			f = j - i;
		}
	}
	return m;
}
  
/* Funzione che calcola la mediana di un array *ordinato*
 * PRECONDIZIONE: array ordinato
 */
double mediana(int a[], int size) {
  int m;
  
  if(size % 2 == 0) {
    m = size / 2;
    return (a[m - 1] + a[m]) / 2.0; // m-1 perche' gli indici partono da 0
  } else {
    m = size / 2; // arrotonda per difetto, abbiamo gia' la posizione della mediana
    return a[m]; 
  }
}
